//+------------------------------------------------------------------+
//|                                                  MyCExtremum.mqh |
//|                                     Copyright 2014, A. Emelyanov |
//|                                        A.Emelyanov2010@yandex.ru |
//+------------------------------------------------------------------+
//|         -, .. |
//|    " " -.           |
//| 1. ZigZag     MetaQuotes;                      |
//| 2. RateZigzag     Zigzag                      |
//| 3. FractalZZ    Zigzag                 |
//+------------------------------------------------------------------+
//|+ 23/12/2014:   .                            |
//+------------------------------------------------------------------+
//|+ 10/11/2014:  FractalsZZ  " ".            |
//+------------------------------------------------------------------+
//|+ 05/11/2014:     Zigzag.      |
//+------------------------------------------------------------------+
//|+ 31/03/2014:      FivePattern.     |
//+------------------------------------------------------------------+
//|! 27/02/2014:     .                    |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, A. Emelyanov"
#property link      "A.Emelyanov2010@yandex.ru"
#property version   "4.10"
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
#define EXTREMUM_RESERVE 0
//--- auxiliary enumeration
enum looling_for
  {
   Start = 0,                       // start seaching
   Pike  = 1,                       // searching for next high
   Sill  = -1                       // searching for next low
  };
//+------------------------------------------------------------------+
//|                                            |
//+------------------------------------------------------------------+
#include <Object.mqh>
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
class MyCExtremum : public CObject
  {
protected:
   //--- ,     
   double            HighMapBuffer[];                             //   " ." 
   double            LowMapBuffer[];                              //   " ."
   double            ZigzagBuffer[];                              //    "ZigZag"
   datetime          TimeBuffer[];                                //    " ZigZag"
public:
                     MyCExtremum();
                    ~MyCExtremum();
   //+---------------------------------------------------------------+
   //|  ZIGZAG  ( )                       |
   //+---------------------------------------------------------------+
   int               GetClassicZigzag(const double &high[],const double &low[],const datetime &time[],
                                      int ExtDepth, int ExtDeviation,int ExtBackstep);
   //+---------------------------------------------------------------+
   //|  ZIGZAG (  + %  )     |
   //+---------------------------------------------------------------+
   int               GetCombiZigzag(const double &high[],const double &low[],const datetime &time[],
                                    int ExtDepth, double ExtDeviation,int ExtBackstep);
   //+---------------------------------------------------------------+
   //| Fractals ZIGZAG                                               |
   //+---------------------------------------------------------------+
   int               GetFractalsZigzag(const double &high[],const double &low[], const datetime &time[],
                                       double ExtDeviation = 1);
   //+---------------------------------------------------------------+
   //|                                                 |
   //+---------------------------------------------------------------+
   int               GetSizeBuffer(void){return(ArraySize(ZigzagBuffer));}//   
   double            GetHighMapBuffer(int i){if(i<ArraySize(HighMapBuffer))return(HighMapBuffer[i]);return(0.0);};//     HighMapBuffer
   double            GetLowMapBuffer(int i){if(i<ArraySize(LowMapBuffer))return(LowMapBuffer[i]);return(0.0);};//     LowMapBuffer
   double            GetZigzagBuffer(int i){if(i<ArraySize(ZigzagBuffer))return(ZigzagBuffer[i]);return(0.0);};//     ZigzagBuffer
   datetime          GetTimeBuffer(int i){if(i<ArraySize(TimeBuffer))return(TimeBuffer[i]);return(0);};//     TimeBuffer
   int               CopyHighMapBuffer(double &in[]){return(ArrayCopy(in,HighMapBuffer));};//   HighMapBuffer
   int               CopyLowMapBuffer(double &in[]){return(ArrayCopy(in,LowMapBuffer));};//   LowMapBuffer
   int               CopyZigzagMapBuffer(double &in[]){return(ArrayCopy(in,ZigzagBuffer));};//   ZigzagBuffer
   int               CopyTimeBuffer(datetime &in[]){return(ArrayCopy(in,TimeBuffer));};//   TimeBuffer
private:
   //---  
   int               GetHighMapZigzag(const double &in[], int ExtDepth, int ExtDeviation, 
                                      int ExtBackstep);           // Classic ZigZag:       
   int               GetLowMapZigzag(const double &in[], int ExtDepth, int ExtDeviation, 
                                      int ExtBackstep);           // Classic ZigZag:    
   int               iHighest(const double &array[], int     depth, 
                              int startPos,    double &delta);    //  
   int               iLowest(const double &array[], int     depth, 
                             int startPos,    double &delta);     //  
  };
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
MyCExtremum::MyCExtremum()
  {
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
MyCExtremum::~MyCExtremum()
  {
  }
//+------------------------------------------------------------------+
//| public  - :                                                |
//+------------------------------------------------------------------+
//|                                     |
//| OUT: -1 ERROR, >0 ALL OK                                         |
//+------------------------------------------------------------------+
int MyCExtremum::GetClassicZigzag(const double   &high[],   //   high
                                  const double   &low[],    //   low
                                  const datetime &time[],   //   
                                  int ExtDepth,             //   ( )
                                  int ExtDeviation,         // " "    
                                  int ExtBackstep           //   ( )
                                 )
  {
   //--- value
   int    shift=0, whatlookfor=0, lasthighpos=0, lastlowpos=0;
   double lasthigh=0.0, lastlow=0.0;
   int    rates_total = ArraySize(time);          //   
   int    limit       = rates_total - ExtDepth;   //   ...
   //+---------------------------------------------------------------+
   //|       !    |
   //+---------------------------------------------------------------+
   if(ArrayIsSeries(high)) ArraySetAsSeries(high,false);
   if(ArrayIsSeries(low))  ArraySetAsSeries(low,false);
   if(ArrayIsSeries(time)) ArraySetAsSeries(time,false);
   //---   
   if(rates_total<20)
     { 
      Print(__FUNCTION__," ERROR: the small size of the buffer.");
      return(-1);                                     
     }
   //--- : Depth and Backstep
   if((ExtDepth < ExtBackstep)||(ExtDepth < 2))
     {
      Print(__FUNCTION__+" ERROR: Is\'not correct a Depth and Backstep. The value of Depth should be greater than Backstep.");
      return(-1);
     }
   //--- : Deviation
   if(ExtDeviation<1 || ExtDeviation>5000)
     { 
      Print(__FUNCTION__," ERROR: Is\'not correct a Deviation. The value of Deviation should be in the interval [1..5000].");
      return(-1);                                     
     }
   //---   ZigzagBuffer[]
   if(ArraySize(ZigzagBuffer)>0) ArrayFree(ZigzagBuffer);               //   
   ArrayResize(ZigzagBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(ZigzagBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(ZigzagBuffer))  ArraySetAsSeries(ZigzagBuffer,  false);
   //---
   if(ArraySize(HighMapBuffer)>0) ArrayFree(HighMapBuffer);             //   
   ArrayResize(HighMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(HighMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(HighMapBuffer)) ArraySetAsSeries(HighMapBuffer, false);
   //---
   if(ArraySize(LowMapBuffer)>0) ArrayFree(LowMapBuffer);               //   
   ArrayResize(LowMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(LowMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(LowMapBuffer))  ArraySetAsSeries(LowMapBuffer,  false);   
   //---
   if(ArraySize(TimeBuffer)>0) ArrayFree(TimeBuffer);                   //   
   ArrayResize(TimeBuffer,     rates_total, EXTREMUM_RESERVE);
   ArrayFill(TimeBuffer,    0, rates_total,   0);
   if(ArrayIsSeries(TimeBuffer))  ArraySetAsSeries(TimeBuffer,  false);   
   //---  ""   
   if(GetHighMapZigzag(high,ExtDepth,ExtDeviation,ExtBackstep) < 0) return(0);
   if(GetLowMapZigzag(low,ExtDepth,ExtDeviation,ExtBackstep)   < 0) return(0);
   //--- final rejection
   for(shift=ExtDepth;shift<rates_total;shift++)
     {
      switch(whatlookfor)
        {
         case Start: // search for peak or lawn
            if(lastlow==0 && lasthigh==0)
              {
               if(HighMapBuffer[shift]!=0)
                 {
                  lasthigh=high[shift];
                  lasthighpos=shift;
                  whatlookfor=Sill;
                  ZigzagBuffer[shift]=lasthigh;
                  TimeBuffer[shift]=time[shift];
                 }
               if(LowMapBuffer[shift]!=0)
                 {
                  lastlow=low[shift];
                  lastlowpos=shift;
                  whatlookfor=Pike;
                  ZigzagBuffer[shift]=lastlow;
                  TimeBuffer[shift]=time[shift];
                 }
              }
            break;
         case Pike: // search for peak
            if(LowMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow && HighMapBuffer[shift]==0.0)
              {
               //---
               ZigzagBuffer[lastlowpos] = 0.0;
               TimeBuffer[lastlowpos]   = 0;
               //---
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
              }
            if(HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]==0.0)
              {
               lasthigh=HighMapBuffer[shift];
               lasthighpos=shift;
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
               whatlookfor=Sill;
              }
            break;
         case Sill: // search for lawn
            if(HighMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh && LowMapBuffer[shift]==0.0)
              {
               //--- 
               ZigzagBuffer[lasthighpos] = 0.0;
               TimeBuffer[lasthighpos]   = 0;
               //---
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
              }
            if(LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]==0.0)
              {
               lastlow=LowMapBuffer[shift];
               lastlowpos=shift;
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
               whatlookfor=Pike;
              }
            break;
         default: 
            return(-1);
        }
     }
   //--- return value of prev_calculated for next call
   return(rates_total);   
  }
//+------------------------------------------------------------------+
//|:  ZIGZAG(  + %  )|
//| OUT: -1 ERROR, >0 ALL OK                                         |
//+------------------------------------------------------------------+
int MyCExtremum::GetCombiZigzag(const double    &high[],     //   high
                                const double    &low[],      //   low
                                const datetime  &time[],     //   
                                int             ExtDepth,    //   ( )
                                double          ExtDeviation,// " ":   + %  
                                int             ExtBackstep  //   ( )
                               )
  {
   //--- value
   int    shift=0, whatlookfor=0, lasthighpos=0, lastlowpos=0, Deviat=1;
   double lasthigh=0.0, lastlow=0.0, percent=0.0;
   int    rates_total = ArraySize(time);          //   
   int    limit       = rates_total - ExtDepth;   //   ...
   //+---------------------------------------------------------------+
   //|       !    |
   //+---------------------------------------------------------------+
   if(ArrayIsSeries(high)) ArraySetAsSeries(high,false);
   if(ArrayIsSeries(low))  ArraySetAsSeries(low,false);
   if(ArrayIsSeries(time)) ArraySetAsSeries(time,false);
   //+---------------------------------------------------------------+
   //|                                      |
   //+---------------------------------------------------------------+
   if(rates_total<20)
     { 
      Print(__FUNCTION__," ERROR: the small size of the buffer.");
      return(-1);                                     
     }
   if(ExtDeviation<0 || ExtDeviation>100)
     { 
      Print(__FUNCTION__," ERROR: Is\'not correct a Deviation. The value of Deviation should be in the interval [0..100].");
      return(-1);                                     
     }
   //--- : Depth and Backstep
   if((ExtDepth < ExtBackstep)||(ExtDepth < 2))
     {
      Print(__FUNCTION__+" ERROR: Is\'not correct a Depth and Backstep. The value of Depth should be greater than Backstep.");
      return(-1);
     }
   //---   ZigzagBuffer[]
   if(ArraySize(ZigzagBuffer)>0) ArrayFree(ZigzagBuffer);               //   
   ArrayResize(ZigzagBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(ZigzagBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(ZigzagBuffer))  ArraySetAsSeries(ZigzagBuffer,  false);
   //---
   if(ArraySize(HighMapBuffer)>0) ArrayFree(HighMapBuffer);             //   
   ArrayResize(HighMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(HighMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(HighMapBuffer)) ArraySetAsSeries(HighMapBuffer, false);
   //---
   if(ArraySize(LowMapBuffer)>0) ArrayFree(LowMapBuffer);               //   
   ArrayResize(LowMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(LowMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(LowMapBuffer))  ArraySetAsSeries(LowMapBuffer,  false);   
   //---
   if(ArraySize(TimeBuffer)>0) ArrayFree(TimeBuffer);                   //   
   ArrayResize(TimeBuffer,     rates_total, EXTREMUM_RESERVE);
   ArrayFill(TimeBuffer,    0, rates_total,   0);
   if(ArrayIsSeries(TimeBuffer))  ArraySetAsSeries(TimeBuffer,  false);   
   //---  Deviation
   if(ExtDeviation < 1)
     {
      Deviat = 1;
     }else
        {
         Deviat = (int)ExtDeviation;
        }
   //---  ""   
   if(GetHighMapZigzag(high,ExtDepth,Deviat,ExtBackstep) < 0) return(0);
   if(GetLowMapZigzag(low,ExtDepth,Deviat,ExtBackstep)   < 0) return(0);
   //--- final rejection
   for(shift=ExtDepth;shift<rates_total;shift++)
     {
      switch(whatlookfor)
        {
         case Start: // search for peak or lawn
            if(lastlow==0 && lasthigh==0)
              {
               if(HighMapBuffer[shift]!=0)
                 {
                  lasthigh=high[shift];
                  lasthighpos=shift;
                  whatlookfor=Sill;
                  ZigzagBuffer[shift]=lasthigh;
                  TimeBuffer[shift]=time[shift];
                 }
               if(LowMapBuffer[shift]!=0)
                 {
                  lastlow=low[shift];
                  lastlowpos=shift;
                  whatlookfor=Pike;
                  ZigzagBuffer[shift]=lastlow;
                  TimeBuffer[shift]=time[shift];
                 }
              }
            break;
         case Pike: // search for peak
            if(LowMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow && HighMapBuffer[shift]==0.0)
              {
               //---
               ZigzagBuffer[lastlowpos] = 0.0;
               TimeBuffer[lastlowpos]   = 0;
               //---
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            //---  ""
            if(LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow)
              {
               //---
               ZigzagBuffer[lastlowpos] = 0.0;
               TimeBuffer[lastlowpos]   = 0;
               //---
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            if(HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]==0.0)
              {
               //--- : %  
               percent = (HighMapBuffer[shift]-lastlow)/(lastlow/100);
               if(percent > ExtDeviation)
                 {
                  lasthigh=HighMapBuffer[shift];
                  lasthighpos=shift;
                  ZigzagBuffer[shift]=lasthigh;
                  TimeBuffer[shift]=time[shift];
                  whatlookfor=Sill;
                 }
               percent = 0.0;
              }            
            break;
         case Sill: // search for lawn
            if(HighMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh && LowMapBuffer[shift]==0.0)
              {
               //--- 
               ZigzagBuffer[lasthighpos] = 0.0;
               TimeBuffer[lasthighpos]   = 0;
               //---
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            if(HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh)
              {
               //--- 
               ZigzagBuffer[lasthighpos] = 0.0;
               TimeBuffer[lasthighpos]   = 0;
               //---
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            if(LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]==0.0)
              {
               //--- : %  
               percent = (lasthigh-LowMapBuffer[shift])/(lasthigh/100);
               if(percent > ExtDeviation)
                 {
                  lastlow=LowMapBuffer[shift];
                  lastlowpos=shift;
                  ZigzagBuffer[shift]=lastlow;
                  TimeBuffer[shift]=time[shift];
                  whatlookfor=Pike;                  
                 }
               percent = 0.0;
              }
            break;
         default: 
            return(-1);
        }
     }
   //--- return value of prev_calculated for next call
   return(rates_total);   
  }
//+------------------------------------------------------------------+
//| Fractals ZIGZAG                                                  |
//| OUT: -1 ERROR, >0 ALL OK                                         |
//+------------------------------------------------------------------+
int MyCExtremum::GetFractalsZigzag(const double    &high[],       //   high
                                   const double    &low[],        //   low
                                   const datetime  &time[],       //  
                                   double          ExtDeviation=1 // " ": %  
                                  )
  {
   //--- value
   int    i=0, shift=0, whatlookfor=0, lasthighpos=0, lastlowpos=0;
   double lasthigh=0.0, lastlow=0.0, percent=0.0;
   int    rates_total = ArraySize(time);          //   
   int    limit       = 2;                        //  
   //+---------------------------------------------------------------+
   //|            |
   //+---------------------------------------------------------------+
   if(ArrayIsSeries(high)) ArraySetAsSeries(high,false);
   if(ArrayIsSeries(low))  ArraySetAsSeries(low,false);
   if(ArrayIsSeries(time)) ArraySetAsSeries(time,false);
   //+---------------------------------------------------------------+
   //|                                      |
   //+---------------------------------------------------------------+
   if(rates_total<20)
     { 
      Print(__FUNCTION__," ERROR: the small size of the buffer.");
      return(-1);                                     
     }
   if(ExtDeviation<0 || ExtDeviation>100)
     { 
      Print(__FUNCTION__," ERROR: Is\'not correct a Deviation. The value of Deviation should be in the interval [0..100].");
      return(-1);                                     
     }
   //+---------------------------------------------------------------+
   //|  value                                                 |
   //+---------------------------------------------------------------+
   if(ArraySize(ZigzagBuffer)>0) ArrayFree(ZigzagBuffer);               //   
   ArrayResize(ZigzagBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(ZigzagBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(ZigzagBuffer))  ArraySetAsSeries(ZigzagBuffer,  false);
   //---
   if(ArraySize(HighMapBuffer)>0) ArrayFree(HighMapBuffer);             //   
   ArrayResize(HighMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(HighMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(HighMapBuffer)) ArraySetAsSeries(HighMapBuffer, false);
   //---
   if(ArraySize(LowMapBuffer)>0) ArrayFree(LowMapBuffer);               //   
   ArrayResize(LowMapBuffer,rates_total, EXTREMUM_RESERVE);
   ArrayFill(LowMapBuffer,0,rates_total,0.0);
   if(ArrayIsSeries(LowMapBuffer))  ArraySetAsSeries(LowMapBuffer,  false);   
   //---
   if(ArraySize(TimeBuffer)>0) ArrayFree(TimeBuffer);                   //   
   ArrayResize(TimeBuffer,     rates_total, EXTREMUM_RESERVE);
   ArrayFill(TimeBuffer,    0, rates_total,   0);
   if(ArrayIsSeries(TimeBuffer))  ArraySetAsSeries(TimeBuffer,  false);   
   //+---------------------------------------------------------------+
   //| 1.  : /    Fractals,|
   //|    . Fractals.mq5                                           |
   //+---------------------------------------------------------------+
   for(i=limit;i<rates_total-3&&!IsStopped();i++)
     {
      //---- Upper Fractal
      if(high[i]>high[i+1] && high[i]>high[i+2] && high[i]>=high[i-1] && high[i]>=high[i-2])
         HighMapBuffer[i]=high[i];
      else HighMapBuffer[i]=0.0;
      //---- Lower Fractal
      if(low[i]<low[i+1] && low[i]<low[i+2] && low[i]<=low[i-1] && low[i]<=low[i-2])
         LowMapBuffer[i]=low[i];
      else LowMapBuffer[i]=0.0;
     }
   //+---------------------------------------------------------------+
   //| 2. final rejection                                            |
   //+---------------------------------------------------------------+
   for(shift=limit;shift<rates_total;shift++)
     {
      switch(whatlookfor)
        {
         case Start: // search for peak or lawn
            if(lastlow==0 && lasthigh==0)
              {
               if(HighMapBuffer[shift]!=0)
                 {
                  lasthigh=high[shift];
                  lasthighpos=shift;
                  whatlookfor=Sill;
                  ZigzagBuffer[shift]=lasthigh;
                  TimeBuffer[shift]=time[shift];
                 }
               if(LowMapBuffer[shift]!=0)
                 {
                  lastlow=low[shift];
                  lastlowpos=shift;
                  whatlookfor=Pike;
                  ZigzagBuffer[shift]=lastlow;
                  TimeBuffer[shift]=time[shift];
                 }
              }
            break;
         case Pike: // search for peak
            if(LowMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow && HighMapBuffer[shift]==0.0)
              {
               //---
               ZigzagBuffer[lastlowpos] = 0.0;
               TimeBuffer[lastlowpos]   = 0;
               //---
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            //---  ""
            if(LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow)
              {
               //---
               ZigzagBuffer[lastlowpos] = 0.0;
               TimeBuffer[lastlowpos]   = 0;
               //---
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            if(HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]==0.0)
              {
               //--- : %  
               percent = (HighMapBuffer[shift]-lastlow)/(lastlow/100);
               if(percent > ExtDeviation)
                 {
                  lasthigh=HighMapBuffer[shift];
                  lasthighpos=shift;
                  ZigzagBuffer[shift]=lasthigh;
                  TimeBuffer[shift]=time[shift];
                  whatlookfor=Sill;
                 }
               percent = 0.0;
              }
            //---  switch
            break;
         case Sill: // search for lawn
            if(HighMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh && LowMapBuffer[shift]==0.0)
              {
               //--- 
               ZigzagBuffer[lasthighpos] = 0.0;
               TimeBuffer[lasthighpos]   = 0;
               //---
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            //---  ""
            if(HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh)
              {
               //--- 
               ZigzagBuffer[lasthighpos] = 0.0;
               TimeBuffer[lasthighpos]   = 0;
               //---
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
               TimeBuffer[shift]=time[shift];
               //--- :  switch
               break;
              }
            if(LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]==0.0)
              {
               //--- : %  
               percent = (lasthigh-LowMapBuffer[shift])/(lasthigh/100);
               if(percent > ExtDeviation)
                 {
                  lastlow=LowMapBuffer[shift];
                  lastlowpos=shift;
                  ZigzagBuffer[shift]=lastlow;
                  TimeBuffer[shift]=time[shift];
                  whatlookfor=Pike;                  
                 }
               percent = 0.0;
              }
            //---  switch
            break;
         default: 
            return(-1);
        }
     }
   //--- return value of prev_calculated for next call
   return(rates_total);   
  }
//+------------------------------------------------------------------+
//| private - :                                                |
//+------------------------------------------------------------------+
//|      array                             |
//+------------------------------------------------------------------+
int MyCExtremum::iHighest(const double &array[],   //    
                          int    depth,            //  
                          int    startPos,         //   
                          double &delta            //  /
                         )
  {
   int index=startPos;
   //--- start index validation
   if(startPos<0)
     {
      Print(__FUNCTION__," ERROR: Invalid parameter in the function iHighest(...)");
      return 0;
     }
   //--- depth correction if need
   if(startPos-depth<0) depth=startPos;
   double max=array[startPos];
   //--- start searching
   for(int i=startPos;i>startPos-depth;i--)
     {
      if(array[i]>max)
        {
         index=i;
         max=array[i];
        }
     }
   //---  delta
   double myMax = array[ArrayMaximum(array,startPos-depth,startPos)];
   double myMin = array[ArrayMinimum(array,startPos-depth,startPos)];
   delta        = myMax - myMin;
   //--- return index of the highest bar
   return(index);
  }
//+------------------------------------------------------------------+
//|       array                             |
//+------------------------------------------------------------------+
int MyCExtremum::iLowest(const double &array[], //    
                         int depth,             //  
                         int startPos,          //   
                         double &delta          //  /
                        )
  {
   int index=startPos;
   //--- start index validation
   if(startPos<0)
     {
      Print(__FUNCTION__," ERROR: Invalid parameter in the function iLowest(...)");
      return 0;
     }
   //--- depth correction if need
   if(startPos-depth<0) depth=startPos;
   double min=array[startPos];
   //--- start searching
   for(int i=startPos;i>startPos-depth;i--)
     {
      if(array[i]<min)
        {
         index=i;
         min=array[i];
        }
     }
   //---  delta
   double myMax = array[ArrayMaximum(array,startPos-depth,startPos)];
   double myMin = array[ArrayMinimum(array,startPos-depth,startPos)];
   delta        = myMax - myMin;
   //--- return index of the lowest bar
   return(index);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
int MyCExtremum::GetHighMapZigzag(const double &in[], //  
                                  int ExtDepth,       //   1
                                  int ExtDeviation,   // ""
                                  int ExtBackstep     //   2
                                 )
  {
   //---  
   int    shift    = 0, back = 0; 
   double lasthigh = 0, res  = 0, val = 0, delt = 0;
   //--- ""    (  ...)
   int    rates_total     = ArraySize(in);            //   
   double deviation       = ExtDeviation * _Point;    //    
   //+---------------------------------------------------------------+
   //|                    Searching High                             |
   //+---------------------------------------------------------------+
   for(shift=ExtDepth;shift<rates_total;shift++)
     {//--- high
      val=in[iHighest(in,ExtDepth,shift,delt)];
      if(val==lasthigh) val=0.0;                      //   
      else
        {
         lasthigh=val;                                //   
         if((val-in[shift])>deviation) val=0.0;
         else
           {
            for(back=1;back<=ExtBackstep;back++)
              {
               res=HighMapBuffer[shift-back];
               if((res!=0) && (res<val)) HighMapBuffer[shift-back]=0.0;
              }
           }
        }
      if(in[shift]==val) HighMapBuffer[shift]=val; else HighMapBuffer[shift]=0.0;
     }   
   //---     
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                   |
//+------------------------------------------------------------------+
int MyCExtremum::GetLowMapZigzag(const double &in[],  //  
                                 int ExtDepth,        //   1
                                 int ExtDeviation,    // ""
                                 int ExtBackstep      //   2
                                )
  {
   //---  
   int    shift = 0, back    = 0; 
   double val   = 0, lastlow = 0, res = 0, delt = 0;  
   //--- ""    (  ...)
   int    rates_total     = ArraySize(in);            //   
   double deviation       = ExtDeviation * _Point;    //    
   //+---------------------------------------------------------------+
   //|                    Searching Low                              |
   //+---------------------------------------------------------------+
   for(shift=ExtDepth;shift<rates_total;shift++)
     {
      val=in[iLowest(in,ExtDepth,shift,delt)];
      if(val==lastlow) val=0.0;
      else
        {
         lastlow=val;
         if((in[shift]-val)>deviation) val=0.0;
         else
           {
            for(back=1;back<=ExtBackstep;back++)
              {
               res=LowMapBuffer[shift-back];
               if((res!=0) && (res>val)) LowMapBuffer[shift-back]=0.0;
              }
           }
        }
      if(in[shift]==val) LowMapBuffer[shift]=val; else LowMapBuffer[shift]=0.0;
     }
   //---     
   return(rates_total);
  }
//+------------------------------------------------------------------+